/* an LP solver for Joho-Kiso   */
/* 2016, Yoshinori Hayakawa     */
/* CITE, Tohoku university      */

#include <math.h>
#include <float.h>
#include <stdio.h>

double A[M+1][N+M+1] ;
double B[M+1] ;

void INIT_COEF(float alpha[][N], float b[], float c[])
{
  int i,j ;

  for (i=0; i<M ; i=i+1) {
    B[i] = b[i] ;
    for (j=0 ; j<N+M+1 ; j=j+1) {
      if (j<N) 
	A[i][j] = alpha[i][j] ;
      else {
	if (j == N+i) A[i][j]= 1.0 ;
	else A[i][j]= 0.0 ;
      }
    }
  }

  B[M]= 0.0 ;
  for (j=0 ; j<N+M+1 ; j=j+1) {
    if (j<N) A[M][j] = -c[j] ;
    else {
      if (j == N+M) A[M][j]= -1 ;
      else A[M][j]=0.0 ;
    }
  }
}

/* An implementation of Algorithm 13 */
int PIVOT()
{
  int i,j,r,s ;

  int found_negative_gamma=0 ;
  for (j=0 ; j<N+M ; j=j+1) {
    if (A[M][j] < 0.0) {
      s= j ;
      found_negative_gamma=1 ;
      break ;
    }
  }

  if (!found_negative_gamma) return 1 ; /* FOUND SOLUTION */

  int found_positive_alpha=0 ;
  for (i=0 ; i<M ; i=i+1) {
    if (A[i][s] > 0.0) found_positive_alpha=1 ;
  }

  if (!found_positive_alpha) return -1 ; /* UNBOUNDED */

  double v = FLT_MAX ;
  for (i=0 ; i<M ; i=i+1) {
    if (A[i][s]>0) {
      if (B[i]/A[i][s] < v) {
	v = B[i]/A[i][s] ;
	r = i ;
      }
    }
  }
  
  double a_rs=A[r][s] ;
  for (j=0; j<N+M ; j=j+1) {
    A[r][j] = A[r][j]/a_rs ;
  }
  B[r] = B[r]/a_rs ;

  for (i=0; i<M+1 ; i=i+1) {
    if (i!=r) {
      double a_is = A[i][s] ;
      for (j=0 ; j<N+M ; j=j+1) {
	A[i][j] = A[i][j] - A[r][j]*a_is ;
      }
      B[i] = B[i] - B[r]*a_is ;
    }
  }

  return 0 ;
}

/* An implementation of Algorithm 14 */
int SIMPLEX()
{
  int res ;

  res = PIVOT() ;
  if (res<0) {
    printf("unbounded\n") ;
    return -1 ;
  } else if (res>0) {
    return 1 ;
  } else {
    return SIMPLEX() ;
  }
}

void DUMP_MATRIX()
{
  int i,j ;
  for (i=0; i<M+1; i=i+1) {
    for (j=0; j<N+M+1; j=j+1) {
      printf("%lf ",A[i][j]) ;
    }
    printf("  %lf ",B[i]) ;
    printf("\n") ;
  }
}

#define EPS (1.0E-6)

float GET_OPT_PARAM(int j)
{
  int i,s ;
  int n_unity=0 ;
  int n_nonzero=0 ;
  for (i=0; i<M; i=i+1) {
    if (fabs(A[i][j]-1.0)<EPS) { n_unity=n_unity+1 ; s=i ; }
    else if (fabs(A[i][j])>EPS) n_nonzero=n_nonzero+1 ;
  }
  if (n_unity==1 && n_nonzero==0) {
    return B[s] ;
  } 
  else 
    return 0 ;
}

float GET_OPT_VALUE()
{
  return B[M] ;
}

void PRINT_SOLUTION()
{
  int j;
  double val ;

  for (j=0; j<N; j=j+1) {
    val = GET_OPT_PARAM(j) ;
    printf("x[%d]= %lf\n",j,val) ;
  }
  printf("z= %lf\n",GET_OPT_VALUE()) ;
}

